<?php

namespace Drp\NativeEventApp\Auth;

use Drp\NativeEventApp\Enums\AuthMethod;
use Drp\NativeEventApp\Exceptions\Contracts\InvalidContactDetail;
use TypeError;

class ContactDetails implements \IteratorAggregate
{
    /**  @var \Illuminate\Support\Collection */
    protected $methods;

    public function __construct(array $contactDetails = [])
    {
        $this->methods = collect($contactDetails)
            ->each(function ($contactDetail): void {
                throw_unless(
                    $contactDetail instanceof ContactDetail,
                    new TypeError('ContactDetails must be instantiated with an array of ContactDetail')
                );
            })
            ->keyBy('method');
    }

    public static function make(array $contactDetails = []): self
    {
        return new self($contactDetails);
    }

    public function has(string $authMethod): bool
    {
        return $this->methods->has($authMethod);
    }

    public function get(string $authMethod): ContactDetail
    {
        return $this->methods->get($authMethod);
    }

    public function getIterator(): iterable
    {
        return $this->methods;
    }

    public function add(AuthMethod $method, ?string $value): self
    {
        try {
            $this->methods[$method->value] = new ContactDetail($method, $value);
        } catch (InvalidContactDetail $exception) { // phpcs:ignore
        }

        return $this;
    }

    public function isEmpty(): bool
    {
        return count($this->methods) === 0;
    }
}
