<?php

namespace Drp\NativeEventApp\Models;

use Drp\NativeEventApp\Quiz\QuestionOperatorProxy;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;

/**
 * @property \Drp\NativeEventApp\Models\Quiz $quiz
 * @property-read \Drp\NativeEventApp\Quiz\QuestionOperatorProxy $operator
 */
class QuizQuestion extends Model
{
    /** @var string  */
    protected $table = 'quiz_questions';

    public $timestamps = false;

    public function quiz(): BelongsTo
    {
        return $this->belongsTo(Quiz::class, 'quiz_id');
    }

    public function answers(): HasMany
    {
        return $this->hasMany(QuizAnswer::class, 'quiz_question_id');
    }

    public function responses(): HasMany
    {
        return $this->hasMany(QuizResponse::class, 'quiz_question_id');
    }

    public function correctAnswers(): HasMany
    {
        return $this->answers()->onlyCorrect();
    }

    public function getOperatorAttribute(): QuestionOperatorProxy
    {
        return new QuestionOperatorProxy($this->quiz, $this);
    }
}
