<?php

namespace Drp\NativeEventApp\Models;

use Drp\NativeEventApp\Contracts\TokenAuthenticatable;
use Drp\NativeEventApp\Models\Concerns\Segmentable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;

class Registration extends Model
{
    use Segmentable;

    /** @var string  */
    protected $table = 'registrations';

    /** @var string[] */
    protected $guarded = [];

    public function event(): BelongsTo
    {
        return $this->belongsTo(Event::class, 'eventId');
    }

    public function device(): BelongsTo
    {
        return $this->belongsTo(Device::class, 'authId', 'auth_identifier');
    }

    public function getFullNameAttribute(): string
    {
        return $this->forename . ' ' . $this->surname;
    }

    public function scopeForEventAndAuthenticatable(
        Builder $query,
        Event $event,
        TokenAuthenticatable $authenticatable
    ): Builder {
        return $query->where('eventId', $event->id)
            ->where('authId', $authenticatable->getId());
    }
}
