<?php

namespace Drp\NativeEventApp\Modules;

use Illuminate\Contracts\Support\Jsonable;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;

abstract class Module implements \JsonSerializable, Jsonable
{
    use HasAttributes, Serializable;

    /** @var array */
    public $styles = [];

    /** @var string[] */
    public $className = [];

    /** @var array */
    public $viewableBy = [];

    /** @var string */
    protected $description;

    /** @var ?Module */
    protected $parent;

    public static function make(): self
    {
        return new static(...func_get_args());
    }

    abstract public static function getDescription(): string;

    public function canHaveChildren(): bool
    {
        return $this instanceof Containerable;
    }

    public function styles(array $styles): self
    {
        $this->styles = $styles;

        return $this;
    }

    /**
     * @param string|array $className
     */
    public function className($className): self
    {
        $this->className = Arr::wrap($className);

        return $this;
    }

    public function viewableBy(Collection $segments): self
    {
        $this->viewableBy = $segments->map->id;

        return $this;
    }
}
