<?php

namespace Drp\NativeEventApp\Test\Api\Auth;

use Drp\NativeEventApp\Test\Api\GraphQLTestCase;
use Drp\NativeEventApp\Test\Support\InteractsWithAuth;
use Laravel\Passport\Bridge\AccessTokenRepository;
use Laravel\Passport\Passport;
use Spatie\Snapshots\MatchesSnapshots;

class AuthTest extends GraphQLTestCase
{
    use MatchesSnapshots, InteractsWithAuth;

    /** @test */
    public function queries_are_protected(): void
    {
        $query = '{
            events(id:1){
                id
            }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function revoked_tokens_cannot_be_used(): void
    {
        $token = $this->createAccessToken('accessToken');
        Passport::token()->first()->update(['revoked' => true]);

        $query = '{
            events(id:1){
                id
            }
        }';

        $headers = ['Authorization' => "Bearer $token"];

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query, $headers));
    }

    /** @test */
    public function revokes_access_token_on_logout(): void
    {
        $token = $this->createAccessToken('accessToken');

        $query = 'mutation {
              logout {
                status
                message
              }
            }';

        $headers = ['Authorization' => "Bearer $token"];

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query, $headers));

        $this->assertTrue(app(AccessTokenRepository::class)->isAccessTokenRevoked($token));
    }
}
