<?php

namespace Drp\NativeEventApp\Test\Support\Builders;

use Closure;
use Drp\NativeEventApp\Models\Device;
use Drp\NativeEventApp\Models\Registration;
use stdClass;

class DeviceBuilder
{
    /** @var int */
    protected $count = 1;

    /** @var Closure|null */
    protected $withRegistrationCallback;

    /** @var \Illuminate\Support\Collection */
    protected $devices;

    /** @var \Illuminate\Support\Collection */
    protected $registrations;

    /** @var bool */
    protected $incrementAuthId = false;

    /** @var int */
    protected $currentAuthId = 1;

    public function __construct()
    {
        $this->devices = collect();
        $this->registrations = collect();
    }

    public function withRegistration(Closure $callback): self
    {
        $this->withRegistrationCallback = $callback;

        return $this;
    }

    public function incrementAuthId(): self
    {
        $this->incrementAuthId = true;

        return $this;
    }

    public function times(int $times): self
    {
        $this->count = $times;

        return $this;
    }

    public function create(): stdClass
    {
        foreach (range(1, $this->count) as $itemCount) {
            $device = factory(Device::class)->state('firebase')->create([
                'auth_identifier' => $this->currentAuthId,
            ]);
            $this->devices->push($device);

            $registration = factory(Registration::class)->create([
                'authId' => $this->currentAuthId,
            ]);

            if ($this->withRegistrationCallback !== null) {
                $callback = $this->withRegistrationCallback;
                $callback($registration, $itemCount);
            }

            if ($this->incrementAuthId) {
                $this->currentAuthId += 1;
            }

            $this->registrations->push($registration->fresh());
        }

        return (object) [
            'devices' => $this->devices,
            'registrations' => $this->registrations,
        ];
    }
}
