<?php

namespace Drp\NativeEventApp\Test\Unit;

use Carbon\Carbon;
use Drp\NativeEventApp\Auth\MagicLinkSigner;
use Drp\NativeEventApp\Exceptions\InvalidAuthIdentifier;
use Drp\NativeEventApp\Test\TestCase;

class MagicLinkSignerTest extends TestCase
{
    /**
     * @var MagicLinkSigner
     */
    protected $signer;

    public function setUp(): void
    {
        $this->signer = new MagicLinkSigner('KEY');
    }

    /**
     * @test
     */
    public function will_sign_url_with_timestamp()
    {
        $signature = $this->signer->sign(1, '1546344000');
        $signatureShouldBe = hash_hmac('sha256', '{"authId":"1","expiry":"1546344000"}', 'KEY');

        $this->assertEquals($signatureShouldBe, $signature);
    }

    /**
     * @test
     */
    public function will_verify_signature()
    {
        $expiry = Carbon::now('+1 day')->timestamp;
        $isValid = $this->signer
            ->verify(
                1,
                $expiry,
                hash_hmac('sha256', '{"authId":"1","expiry":"' . $expiry . '"}', 'KEY')
            );

        $this->assertTrue($isValid);
    }

    /**
     * @test
     */
    public function throws_exception_if_invalid()
    {
        try {
            $expiry = Carbon::now('-1 day')->timestamp;
            $this->signer
                ->verify(
                    1,
                    $expiry,
                    'INVALID'
                );
        } catch (InvalidAuthIdentifier $exception) {
            $this->assertEquals(InvalidAuthIdentifier::INVALID_SIGNATURE, $exception->getCode());
            return;
        }

        $this->fail('Exception not thrown');
    }

    /**
     * @test
     */
    public function throws_exception_if_expired()
    {
        try {
            $expiry = Carbon::now()->subDay()->timestamp;
            $this->signer
                ->verify(
                    1,
                    $expiry,
                    hash_hmac('sha256', '{"authId":"1","expiry":"' . $expiry . '"}', 'KEY')
                );

        } catch (InvalidAuthIdentifier $exception) {
            $this->assertEquals(InvalidAuthIdentifier::EXPIRED, $exception->getCode());
            return;
        }
        $this->fail('Exception not thrown');
    }
}
